#!/bin/bash

set -e
set -x

export LC_ALL=C LANG=C

### Make sure the following pathnames reflect your installation.

LOCALMAXS="/home/shared/tools/LocalMaxs/"
mwetk() { python /home/shared/tools/mwetoolkit/bin/"$@"; }
nsp() { perl "/home/shared/tools/Text-NSP-1.23/bin/$@"; }
ucs() { "/home/shared/tools/UCS-0.6/System/perl/bin/$@"; }

mkdir times localmaxs mwetk mwetk/indices nsp ucs || true
ln -s /home/shared/tools/mwetoolkit/dtd localmax/dtd || true
ln -s /home/shared/tools/mwetoolkit/dtd mwetk/dtd || true
ln -s /home/shared/tools/mwetoolkit/dtd nsp/dtd || true
ln -s /home/shared/tools/mwetoolkit/dtd ucs/dtd || true

###

do-eval() {
	mwetk eval_automatic.py -c "$@"
}


# Regular expressions for selecting noun compounds and verb-particle candidates
# for NSP and multiwords (LocalMaxs).

nsp_filter_en_noun='^([^/]+/[NA]<>)*[^/]+/N<>[0-9]+ '
nsp_filter_en_verb='^[^/]+/V[A-Z]*<>([^<]*<>)*(up|down|away|off|out|in|on)/[A-Z]+<>[0-9]+ '
nsp_filter_fr_noun='^[^/]+/NOM[:A-Z]*<>([^/]+/(NOM|ADJ)[:A-Z]*<>|(de|à|en)/[A-Z]+<>([^/]*/DET[:A-Z]*<>)?[^/]+/NOM[:A-Z]*<>)+[0-9]+ '

multiwords_filter_en_noun=$'^([^/]+/[JNA] )*[^/]+/N\t'
multiwords_filter_en_verb=$'^[^/]+/V[A-Z]*([^\t ]* | )([^/]+/[^V(PCT)][A-Z]* )*(up|down|away|off|out|in|on)/[A-Z]+\t'
multiwords_filter_fr_noun=$'^[^/]+/NOM[:A-Z]*( [^/]+/(NOM|ADJ)[:A-Z]*| (de|à|en)/[A-Z]+( [^/]*/DET[:A-Z]*)? [^/]+/NOM[:A-Z]*)+\t'

multiwords_sed_en_verb='/^(be|have)\//d; s/^([^ ]*)[^\t]* ([^\t]*)\t/\1 \2\t/; s/\/R/\/P/g'

# Execute an action and save its execution time.
dotime() {
	local name="$1"; shift
	printf "\e[1m%s\e[0m " "$name" >&2
	if [[ -f times/$name ]]; then
		# If the action has been already executed, skip it.
		printf "\e[1;33mskipped!\e[0m\n"
	else
		printf "\n\e[1;32m%s\e[0m\n" "$*"
		SECONDS=0
		eval "$*"
		echo "$SECONDS $*" >>"times/$name"
		printf "Finished in \e[1m%s\e[0m seconds.\n" "$SECONDS"
	fi
}

######### CORPUS CONSTRUCTION/CONVERSION

# Build small, medium and large corpus from the full Europarl corpus.
for lang in en fr; do
	for size in 5k 50k 500k; do
		[[ -f corpus/corpus-$lang-$size.xml ]] ||
			mwetk head.py -n ${size%k}000 corpus/corpus-$lang.xml >corpus/corpus-$lang-$size.xml

		dotime multiwords-00-corpus2txt-$lang-$size \
			$'mwetk xml2csv.py -p corpus/corpus-$lang-$size.xml | awk \'BEGIN{ FS = "\\t"}{ print $2 }\' > corpus/corpus-$lang-$size.txt'
		[[ -s corpus/corpus-$lang-$size.txt ]] || { echo "Empty!!"; exit 2; }
	done
done


######### REFS

# Build the references for each generated corpus from the full reference.
# This is so that the references only contain entries that can actually be
# extracted from each corpus. Reference entries occurring less than twice
# in the corresponding corpus are eliminated.

#for size in 5k 50k 500k; do
#	for lang in en fr; do
#		[[ -f refs/ref-$lang-noun-$size.xml ]] ||
#		mwetk counter.py -g -i mwetk/indices/corpus-${lang}-${size} refs/ref-${lang}-noun.xml |
#		mwetk filter.py -t corpus-${lang}-${size}:2 > refs/ref-${lang}-noun-${size}.xml;
#	done
#	[[ -f refs/ref-en-verb-$size.xml ]] ||
#	mwetk counter.py -g -i mwetk/indices/corpus-en-${size} refs/ref-en-verb.xml | 
#	grep -v "pos=\"V\" ><freq name=\"corpus-en-5k\" value=\"[01]\"" > refs/ref-en-verb-${size}.xml
#done


# References containing only bigrams (used for UCS) and only n-grams of length
# at most 4 (used by NSP).

for langtype in en-noun en-verb fr-noun; do
	for size in 5k 50k 500k; do
		for pat in 2gram max4gram; do
			[[ -f refs/ref-$langtype-$size-$pat.xml ]] ||
				mwetk filter.py -p patterns/$pat.xml refs/ref-$langtype-$size.xml >refs/ref-$langtype-$size-$pat.xml ||
					{ rm refs/ref-$langtype-$size-$pat.xml; exit 2; } # Clean up if error occurs.
		done
	done
done

########### 2. LOCALMAXS

# 1) Extract the candidates

for lang in en fr; do
	for size in 5k 50k; do
		for mode in strict relaxed; do
			dotime multiwords-01-extract-$lang-$mode-$size \
				"python3 \$LOCALMAXS/multiwords-$mode-rev12/multiwords.py scp 5 < corpus/corpus-$lang-$size.txt > localmaxs/cands-$lang-$mode-$size.txt"
		done
	done

	dotime multiwords-01-extract-$lang-strict-500k \
		"bash $LOCALMAXS/multiwords-strict-bigcorpus-rev5/multiwords.sh scp 5 300M < corpus/corpus-$lang-500k.txt > localmaxs/cands-$lang-strict-500k.txt"
done


# 2) Convert output to XML, keeping frequency and glue information

# 3) Filter based on nominal and verbal expressions, using the filter.py script of mwetoolkit and the pattern files. 
# We need to remove intervening materials in verbal expressions
# After the filter, we can remove POS information
# uniq in the end

# 4) Calculate precision and recall

# 5) Calculate mean average precision

for lang in en fr; do ### TODO: fr
	for size in 5k 50k 500k; do
		for mode in strict relaxed; do
			prefix="localmaxs/cands-$lang-$mode-$size"
			[[ -f $prefix.txt ]] || continue
	

			types=(noun)
			[[ $lang = en ]] && types+=(verb)
			for type in "${types[@]}"; do
			
				echo "Filtering/XMLifying $prefix"
				if [[ $type = verb ]]; then
					dotime multiwords-02-candidates2xml-$lang-$mode-$size-$type \
						"cat $prefix.txt | grep -E \"\$multiwords_filter_${lang}_${type}\" | sed -re \"\$multiwords_sed_en_verb\" | sort -t$'\t' -k 1,1 | awk -f localmaxs-uniq.awk | ./localmaxs2xml.sh > $prefix-$type.xml"

				else

					# Select only noun compounds, then filter out those occurring only once.
					dotime multiwords-02-candidates2xml-$lang-$mode-$size-$type \
						"grep -E \"\$multiwords_filter_${lang}_${type}\" -- $prefix.txt | grep -v $'^[^\t]*\t1\t' | ./localmaxs2xml.sh > $prefix-$type.xml"
				fi

				pattern="patterns/pattern-$lang-$type.xml"
				[[ $pattern ]] || continue

				echo "Evaluating $prefix"
				evalopts=""; [[ $lang = fr ]] && evalopts='-L'
				dotime multiwords-05-eval-$lang-$mode-$size-$type \
					"do-eval -g $evalopts -r refs/ref-$lang-$type-$size.xml $prefix-$type.xml >localmaxs/eval-$lang-$mode-$size-$type.xml 2>localmaxs/eval-$lang-$mode-$size-$type-stats.xml"
				[[ -s "localmaxs/eval-$lang-$mode-$size-$type.xml" ]] || { echo "Empty!"; exit 1; }
	
				echo "Computing MAP for $prefix"
				dotime multiwords-06-map-$lang-$mode-$size-$type \
					"mwetk map.py -f glue localmaxs/eval-$lang-$mode-$size-$type.xml >localmaxs/map-$lang-$mode-$size-$type.xml"
			done
		done
	done
done


######### MWETOOLKIT

# 1) Index

for lang in en fr; do
	for size in 5k 50k 500k; do
		dotime mwetk-00-index-$lang-$size "mwetk index.py -v -a lemma:pos -i mwetk/indices/corpus-$lang-$size corpus/corpus-$lang-$size.xml"
	done
done

# 2) Extract, then filter (for verbal), count and calculate association measures

# verbal expressions
for size in 5k 50k 500k; do
	thresh=2
	#[[ $size = 500k ]] && thresh=5
	dotime mwetk-01-en-$size-verb-candidates "mwetk candidates.py -v -f -g -p patterns/pattern-en-verb.xml corpus/corpus-en-$size.xml   > mwetk/cand-en-verb-$size.xml"
	dotime mwetk-02-en-$size-verb-restofpipe "
		mwetk filter.py -p patterns/filter-be-have.xml -v -r mwetk/cand-en-verb-$size.xml | 
		mwetk counter.py -i mwetk/indices/corpus-en-$size -g -v -J | 
		mwetk filter.py -v -t corpus-en-$size:$thresh |
		mwetk feat_association.py -m mle:dice:t:pmi > mwetk/assoc-en-verb-$size.xml
	"
done

# nominal expressions
for lang in en fr; do
	for size in 5k 50k 500k; do
	    thresh=2
	    #[[[ $size = 500k ]] && thresh=3
	    dotime mwetk-01-$lang-$size-noun-candidates "mwetk candidates.py -v -g -p patterns/pattern-$lang-noun.xml corpus/corpus-$lang-$size.xml   > mwetk/cand-$lang-noun-$size.xml"
	
	    dotime mwetk-02-$lang-$size-noun-restofpipe "
			mwetk counter.py -i mwetk/indices/corpus-$lang-$size -g -v mwetk/cand-$lang-noun-$size.xml | 
		    mwetk filter.py -v -t corpus-$lang-$size:$thresh |
			mwetk feat_association.py -m mle:dice:t:pmi > mwetk/assoc-$lang-noun-$size.xml
		"
		[[ -s mwetk/assoc-$lang-noun-$size.xml ]] || { echo "Empty!" >&2; exit 2; }
	done
done

# 4) Calculate precision and recall
# 5) Calculate mean average precision for each measure

for lang in en fr; do
	for size in 5k 50k 500k; do
		types=(noun)
		[[ $lang = en ]] && types+=(verb)
	    for type in "${types[@]}"; do
	        set -- {mle,dice,t,pmi}_corpus-$lang-$size
	        features="$(sed 's/ /:/g' <<<"$*")" # Lamentável.
	
			evalopts=""; [[ $lang = fr ]] && evalopts='-L'
	        dotime mwetk-03-eval-$lang-$size-$type \
				"do-eval -g $evalopts -r refs/ref-$lang-$type-$size.xml mwetk/assoc-$lang-$type-$size.xml > mwetk/eval-$lang-$type-$size.xml 2> mwetk/eval-$lang-$type-$size-stats.txt"
	        dotime mwetk-04-map-$lang-$size-$type \
				"mwetk map.py -f $features mwetk/eval-$lang-$type-$size.xml >mwetk/map-$lang-$type-$size.xml"
	    done
	done
done


######### TEXT::NSP

### Extract noun compounds.

for lang in en fr; do
	for size in 5k 50k 500k; do
		ngram_lengths="2 3 4"; [[ $size = 500k ]] && ngram_lengths="2 3"

		for n in $ngram_lengths; do ## No assoc measure > 4
			[[ $n -ge 4 && $size = 500k ]] && {
				echo -e "\e[1;31mSKIPPING ${n}grams for 500k due to lack of memory\e[0m"
				continue
			}
			dotime nsp-01-count-noun-$lang-$size-${n}gram \
				"nsp count.pl --frequency 2 --newLine --ngram $n --token patterns/general-tokens.txt - corpus/corpus-$lang-$size.txt | tee nsp/raw-noun-$lang-$size-${n}gram.txt | ./filter-nsp.sh \"\$nsp_filter_${lang}_noun\" >nsp/count-noun-$lang-$size-${n}gram.txt"
			case "$n" in
				2) measures=(ll pmi ps tmi) ;;
				3) measures=(ll pmi ps tmi) ;;
				4) measures=(ll) ;;
				*) echo "Oops!"; exit 2 ;;
			esac

			for measure in "${measures[@]}"; do
				dotime nsp-02-assoc-noun-$lang-$size-${n}gram-$measure \
					"nsp statistic.pl --ngram $n $measure nsp/assoc-noun-$lang-$size-${n}gram-$measure.txt nsp/count-noun-$lang-$size-${n}gram.txt"
			done

		done


		dotime nsp-03-assoc2xml-noun-$lang-$size \
			"./nsp-assoc2xml.sh noun $lang $size gram $ngram_lengths >nsp/assoc-noun-$lang-$size.xml"

		evalopts=""; [[ $lang = fr ]] && evalopts='-L'
		dotime nsp-04-eval-noun-$lang-$size \
			"do-eval -g $evalopts -r refs/ref-$lang-noun-$size-max4gram.xml nsp/assoc-noun-$lang-$size.xml >nsp/eval-noun-$lang-$size.xml 2>nsp/eval-noun-$lang-$size-stats.txt"
			evalopts=""; [[ $lang = fr ]] && evalopts='-L'

		#features="$(sed 's/ /:/g' <<<"${measures[*]}")"

		dotime nsp-05-map-noun-$lang-$size \
			"mwetk map.py -f ll:pmi:ps:tmi nsp/eval-noun-$lang-$size.xml >nsp/map-noun-$lang-$size.txt"
			

	done
done

### Extract verb compounds.

for lang in en; do
	for size in 5k 50k 500k; do
		for window in 5; do
			dotime nsp-01-count-verb-$lang-$size-${window}window \
				"nsp count.pl --frequency 2 --newLine --ngram 2 --stop patterns/$lang-verbal-stopwords.txt --window $window --token patterns/general-tokens.txt - corpus/corpus-$lang-$size.txt | ./filter-nsp.sh \"\$nsp_filter_en_verb\" >nsp/count-verb-$lang-$size-${window}window.txt"

			measures=(ll pmi ps tmi)

			for measure in "${measures[@]}"; do
				dotime nsp-02-assoc-verb-$lang-$size-${window}window-$measure \
					"nsp statistic.pl $measure nsp/assoc-verb-$lang-$size-${window}window-$measure.txt nsp/count-verb-$lang-$size-${window}window.txt"

			done

			dotime nsp-03-assoc2xml-verb-$lang-$size-${window}window \
				"./nsp-merge-assocs-old.sh nsp/assoc-verb-$lang-$size-${window}window-* >nsp/assoc-verb-$lang-$size-${window}window.xml"
				
			evalopts=""; [[ $lang = fr ]] && evalopts='-L'
			dotime nsp-04-eval-verb-$lang-$size-${window}window \
				"do-eval -g $evalopts -r refs/ref-$lang-verb-$size-max4gram.xml nsp/assoc-verb-$lang-$size-${window}window.xml >nsp/eval-verb-$lang-$size-${window}window.xml 2>nsp/eval-verb-$lang-$size-${window}window-stats.txt"

			#features="$(sed 's/ /:/g' <<<"${measures[*]}")"

			dotime nsp-05-map-verb-$lang-$size-${window}window \
    			"mwetk map.py -f ll:pmi:ps:tmi nsp/eval-verb-$lang-$size-${window}window.xml >nsp/map-verb-$lang-$size-${window}window.txt"
		done
	done
done


##### UCS

ucs_measures=(
	am.random
	am.z.score.corr
	am.chi.squared.corr
	am.Poisson.pv
	am.Fisher.pv
	am.MI
	am.relative.risk
	am.odds.ratio.disc
	am.MS
	am.gmean
	am.local.MI
)

unset IFS
ucs_measure_colon="$(sed 's/ /:/g' <<<"${ucs_measures[*]}")"

for langtype in en-noun en-verb fr-noun; do
	lang="${langtype%%-*}"
	for size in 5k 50k 500k; do
		# Use 2-grams from mwetoolkit's candidate list.
		dotime ucs-00-assoc2ucs-$langtype-$size \
			"mwetk xml2ucs.py mwetk/assoc-$langtype-$size.xml >ucs/pairs-$langtype-$size.ds"

		dotime ucs-01-addmeasures-$langtype-$size \
			"ucs ucs-add ${ucs_measures[*]} TO ucs/pairs-$langtype-$size.ds INTO ucs/assoc-$langtype-$size.ds"

		dotime ucs-02-assoc2xml-$langtype-$size \
			"./ucs-assoc2xml.sh <ucs/assoc-$langtype-$size.ds | mwetk filter.py -t 2 >ucs/assoc-$langtype-$size.xml"

		evalopts=""; [[ $lang = fr ]] && evalopts='-L'
		dotime ucs-03-eval-$langtype-$size \
			"do-eval -g $evalopts -r refs/ref-$langtype-$size-2gram.xml ucs/assoc-$langtype-$size.xml >ucs/eval-$langtype-$size.xml 2>ucs/eval-$langtype-$size-stats.txt"
				[[ -s "ucs/eval-$langtype-$size.xml" ]] || { echo "Empty!"; exit 2; }
	
		dotime ucs-04-map-$langtype-$size \
			"mwetk map.py -f $ucs_measure_colon ucs/eval-$langtype-$size.xml >ucs/map-$langtype-$size.txt"
		[[ -s "ucs/map-$langtype-$size.txt" ]] || { echo "Empty!"; exit 2; }
	done
done
